/*
	Various global databases by cat02e@fsu.edu
*/

#pragma once
#ifndef BOTDB_H
#define BOTDB_H

#include "hostlist.h"
#include "player.h"
#include "botinfo.h"

class opEntry
{
	Uint32 loginCount;		// Overall count of logins, this instantiation
	Uint32 failedAttempts;	// Count of failed logins, this instantiation
	Operator_Level access;
	String name, pass;

	friend struct BOT_DATABASE;

public:
	opEntry(char *nname, char *ppass, Operator_Level aaccess);

	void setPassword(char *ppass);
	void setName(char *nname);
	void setAccess(Operator_Level aaccess);

	Operator_Level getAccess();

	void addCounter();
	void addFailure();
	Uint32 getOverallCount();
	Uint32 getFailureCount();
	char *getName();

	bool validateName(char *nname);
	bool validatePass(char *ppass);
};

class cmdAlias
{
	String cmd, alias;

public:
	cmdAlias(char *ccmd, char *aalias);

	bool isCmd(char *ccmd);
	bool isAlias(char *aalias);

	bool test(char *&ccmd);	// change command to alias on match

	String &getAlias();
	String &getCommand();
};

struct BOT_DATABASE
{
	// Database
	char path[532];

	BOT_DATABASE();

	Uint32 lastSave;			// in hundredths of a second

	// Parameters
	BOT_INFO botInfo;

	// Spawns
	hostList spawns;

	void loadSpawns();

	// Operators
	_linkedlist <opEntry> opList;
	bool operatorsUpdated;

	opEntry *findOperator(char *name);
	opEntry *addOperator(char *name, Operator_Level level);
	opEntry *addOperator(char *name, char *pass, Operator_Level level);
	bool removeOperator(char *name);

	void loadOperators();
	void saveOperators();

	// Aliasing
	_linkedlist <cmdAlias> aliasList;
	bool aliasesUpdated;

	void aliasCommand(char *&command);
	void addAlias(char *command, char *alias);
	bool killAlias(char *alias);

	cmdAlias *findAlias(char *alias);

	void loadAliases();
	void saveAliases();

	String getAliasList(char *command);

	// INI
	char regName[40];
	char regEMail[40];
	char regState[40];
	BYTE regAge;
	Uint32 loginIP;
	Uint16 loginPort;
	bool recordLogins;
	Uint16 resX;
	Uint16 resY;
	bool chatterLog;
	bool encryptMode;
	char windowCaption[64];
	bool maskBan;
	bool remoteInterpreter;
	bool remoteOperator;
	bool playerVoices;
	Uint32 saveInterval;			// in hundredths of a second
	char spawnsFile[64];
	char commandsFile[64];
	char operatorsFile[64];
	Uint32 maxSpawns;
	bool noTerminal;
	bool forceContinuum;	// hidden option
	bool runSilent;			// hidden option
	bool disablePub;
	bool noisySpectator;

	void reloadINI(bool doLogin);	// doLogin: Ignore [Login] section to avoid DoS attempts
};

#endif	// BOTDB_H
